/***************************************************************************
                          cachablepixmaparray.h  -  description
                             -------------------
    begin                : Fri Oct 25 2002
    copyright            : (C) 2002, 2004 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CACHABLEPIXMAPARRAY_H
#define CACHABLEPIXMAPARRAY_H

#include <qcanvas.h>
#include <qstring.h>
#include <qmap.h>

#include "tpbase.h"

/** Extends the QCanvasPixmapArray by a reference counter, so that it can safely be shared
	across multiple animations, without being accidentally deleted.
	You do NOT want to call the constructor/destructor by hand. Use the
	static functions create () and discard () instead!
  *@author Thomas Friedrichsmeier
  */

class CachablePixmapArray : public QCanvasPixmapArray, TPBase {
public: 
/** Usually you do NOT want call the constructor manually.
	It actually constructs the pixmaparray without prior consultation
	of the cache. Use the static
	CachablePixmapArray::create instead, which will create (and return)
	a new Array if appropriate, or else an existing one. */
	CachablePixmapArray();
/** You should NEVER "delete" a CachablePixmapArray by hand. Instead,
	use the static CachablePixmapArray::discard. It has essentially the
	same effect as a normal destructor, but first checks, whether there
	are further instances of this array still in use. */
	~CachablePixmapArray();
/** Clears the cache of PixmapArrays. Should be called, whenever it
	is unlikely, that further matches will be found in the cache, in order to free
	used memory. (E.g. between levels) */
	static void resetCache ();
/** Creates a new CachablePixmapArray, or returns a pointer from cache,
	if there is one.
	This is essentially the "constructor" for this class */
	static CachablePixmapArray* create (const QString & datafilenamepattern, int fc=0, int def_width=20, int def_height=20, bool preserve_alpha=false);
/** Delete a CachablePixmapArray, making sure it only really get deleted,
	if it is no longer used anywhere.
	This is essentially the "destructor" for this class */
	static void discard (CachablePixmapArray *array);
/** Simply returns a pointer to the given CachablePixmapArray (after
	increasing the reference count! */
	static CachablePixmapArray* copy (CachablePixmapArray *array);
private:
/** Keeps track of how many more times this PixmapArray is in use */
	int references;
	typedef QMap <QString, CachablePixmapArray *> CacheMap;
/** The map used for caching */
	static CacheMap cache_map;
/** Internal procedure called, if there is no match in the cache (i.e.
	the animation actually has to be read) */
	bool build (const QString & datafilenamepattern, int fc, bool preserve_alpha);
/** "builds" an empty pixmap array (used if normal build fails */
	void buildEmpty (int width, int height);
};

#endif
